-------------------------------------------------------------
-- Memory Grant Feedback on Row Store - SQL Server 2019
-- DATAFLY - 202006
-------------------------------------------------------------

USE [AdventureWorksBig]
GO
ALTER DATABASE AdventureWorksBig SET COMPATIBILITY_LEVEL = 150
SELECT compatibility_level FROM sys.databases WHERE name = 'AdventureWorksBig'
GO
-- Ignorer les rsultats...

-- Exemple : trop de mmoire alloue, ressources gaspilles
ALTER DATABASE SCOPED CONFIGURATION CLEAR PROCEDURE_CACHE;  
GO
-- On relance plusieurs fois la requte : Memory grant feedback, la mmoire diminue
SELECT SOH.SalesOrderID,
       SOH.RevisionNumber,
       SOH.OrderDate,
       SOH.Status,
       SOH.PurchaseOrderNumber,
       SOH.CustomerID,
       SODR.SalesOrderDetailID,
       SODR.OrderQty,
       SODR.ProductID,
       SODR.UnitPrice,
       SODR.UnitPriceDiscount
	   FROM Sales.SalesOrderHeader AS SOH
JOIN Sales.SalesOrderDetailRedist AS SODR ON SODR.SalesOrderID = SOH.SalesOrderID
ORDER BY SOH.Comment
GO 3

-- Exemple : mmoire alloue trop faible : spill dans tempdb, la mmoire augmente
-- On fausse l'estimation pour la dmo : 50000 lignes au lieu de 5 Millions
UPDATE STATISTICS Sales.SalesOrderDetailRedist WITH ROWCOUNT=50000
ALTER DATABASE SCOPED CONFIGURATION CLEAR PROCEDURE_CACHE;  
GO
-- On relance plusieurs fois la requte : Memory grant feedback
SELECT SOH.SalesOrderID,
       SOH.RevisionNumber,
       SOH.OrderDate,
       SOH.Status,
       SOH.PurchaseOrderNumber,
       SOH.CustomerID,
       SODR.SalesOrderDetailID,
       SODR.OrderQty,
       SODR.ProductID,
       SODR.UnitPrice,
       SODR.UnitPriceDiscount
	   FROM Sales.SalesOrderHeader AS SOH
JOIN Sales.SalesOrderDetailRedist AS SODR ON SODR.SalesOrderID = SOH.SalesOrderID
ORDER BY SOH.Comment
GO 3

-- Retour  l'tat normal
UPDATE STATISTICS Sales.SalesOrderDetailRedist WITH ROWCOUNT=5000000
ALTER DATABASE SCOPED CONFIGURATION CLEAR PROCEDURE_CACHE;  

-- Activer ou dsactiver 
ALTER DATABASE SCOPED CONFIGURATION SET ROW_MODE_MEMORY_GRANT_FEEDBACK = ON;  -- ou OFF

-- desactiver feedback batch mode ( partir de SQL 2017)
ALTER DATABASE SCOPED CONFIGURATION SET DISABLE_BATCH_MODE_MEMORY_GRANT_FEEDBACK = ON; -- SQL Server 2017
ALTER DATABASE SCOPED CONFIGURATION SET BATCH_MODE_MEMORY_GRANT_FEEDBACK = OFF;  -- SQL 2019 / Azure