-------------------------------------------------------------
-- Automatic Plan Correction - SQL Server 2017 
-- DATAFLY - 202006
-------------------------------------------------------------
USE Orders;
GO
SELECT COUNT(*) FROM SalesOrderDetailStatus
SELECT Status,COUNT(*) AS NbLignes FROM SalesOrderDetailStatus GROUP BY Status ORDER BY Status
-- Compter le nombre de valeurs distinctes (SQL 2019)
SELECT APPROX_COUNT_DISTINCT(Status) AS Nb_Distinct_Values FROM SalesOrderDetailStatus
-- voyons les index
EXEC sp_showindex 'SalesOrderDetailStatus'

-- Activer le Query Store
ALTER DATABASE current 
SET QUERY_STORE = ON
    ( 
      OPERATION_MODE = READ_WRITE   
    , CLEANUP_POLICY = ( STALE_QUERY_THRESHOLD_DAYS = 5 ) 
    , DATA_FLUSH_INTERVAL_SECONDS = 900  
    , MAX_STORAGE_SIZE_MB = 10  
    , INTERVAL_LENGTH_MINUTES = 1  
    ); 
-- Activer Automatic Plan Correction
ALTER DATABASE current
SET AUTOMATIC_TUNING ( FORCE_LAST_GOOD_PLAN = ON );
-- DMV
SELECT *
FROM   sys.database_automatic_tuning_options;
GO

-- Vider le cache de procdures et le buffer
ALTER DATABASE SCOPED CONFIGURATION CLEAR PROCEDURE_CACHE;
DBCC DROPCLEANBUFFERS;
GO

-- Enregistrer le plan pour 49.99 M lignes dans le Query Store -> column store
EXECUTE sp_executesql @stmt = N'SELECT COUNT(*) FROM SalesOrderDetailStatus WHERE Status = @status', 
@params = N'@status smallint', @status = 1;
GO 50

-- Vider le cache de procdures et le buffer
ALTER DATABASE SCOPED CONFIGURATION CLEAR PROCEDURE_CACHE;
DBCC DROPCLEANBUFFERS;
GO

-- Lancer pour une seule ligne : index search
EXECUTE sp_executesql @stmt = N'SELECT COUNT(*) FROM SalesOrderDetailStatus WHERE Status = @status'
, @params = N'@status smallint', @status = 0;
GO

DBCC DROPCLEANBUFFERS;  -- uniquement le buffer de donnes

-- Lancer pour 49.99 M lignes
EXECUTE sp_executesql @stmt = N'SELECT COUNT(*) FROM SalesOrderDetailStatus WHERE Status = @status'
, @params = N'@status smallint', @status = 1;
GO 15

-- DMV
SELECT * FROM sys.dm_db_tuning_recommendations

-- See the tuning recommendations
SELECT  reason,
        score,
        script = JSON_VALUE(details, '$.implementationDetails.script'),
        planForceDetails.[query_id],
        planForceDetails.[new plan_id],
        planForceDetails.[recommended plan_id],
        estimated_gain = (regressedPlanExecutionCount+recommendedPlanExecutionCount)*(regressedPlanCpuTimeAverage-recommendedPlanCpuTimeAverage)/1000000,
        error_prone = IIF(regressedPlanErrorCount>recommendedPlanErrorCount, 'YES','NO')
FROM   sys.dm_db_tuning_recommendations
CROSS APPLY OPENJSON (Details, '$.planForceDetails')
                WITH (  [query_id] int '$.queryId',
                        [new plan_id] int '$.regressedPlanId',
                        [recommended plan_id] int '$.recommendedPlanId',
                        regressedPlanErrorCount int,
                        recommendedPlanErrorCount int,
                        regressedPlanExecutionCount int,
                        regressedPlanCpuTimeAverage float,
                        recommendedPlanExecutionCount int,
                        recommendedPlanCpuTimeAverage float ) as planForceDetails;

-- cleanup
ALTER DATABASE current
SET AUTOMATIC_TUNING ( FORCE_LAST_GOOD_PLAN = OFF );
ALTER DATABASE Orders SET QUERY_STORE CLEAR;
ALTER DATABASE [Orders] SET QUERY_STORE = OFF;
GO
