---------------------------------------------
-- Scalar function inlining - SQL Server 2019
-- DATAFLY - 202006
-- Calculez l'age des personnes
---------------------------------------------
USE Persons
GO
SET STATISTICS TIME ON
GO
--SELECT compatibility_level FROM sys.databases WHERE name = 'Persons'  --> 150
--GO


--1. Premire syntaxe : non inlineable
CREATE OR ALTER FUNCTION fn_get_age (@DTE_NAI date)
RETURNS INT 
AS BEGIN
 RETURN DATEDIFF(YEAR, @DTE_NAI, CURRENT_TIMESTAMP) +
       CASE WHEN MONTH(@DTE_NAI) > MONTH(CURRENT_TIMESTAMP) THEN -1
	        WHEN MONTH(@DTE_NAI) < MONTH(CURRENT_TIMESTAMP) THEN 0
			WHEN DAY(@DTE_NAI) > DAY(CURRENT_TIMESTAMP) THEN -1
			ELSE 0
	   END 
END
GO

-- La fonction est-elle inlinable ?
SELECT OBJECT_NAME(object_id) name,
       is_inlineable
FROM sys.sql_modules
WHERE OBJECTPROPERTY(object_id, 'IsScalarFunction') = 1;
GO


--2. Seconde syntaxe : inlineable
CREATE OR ALTER FUNCTION fn_get_age2 (@DTE_NAI DATE, @DTE_JOUR DATE)
RETURNS INT 
-- WITH INLINE = ON  (c'est par dfaut en compatibilit 150)
AS BEGIN
 RETURN DATEDIFF(YEAR, @DTE_NAI, @DTE_JOUR) +
       CASE WHEN MONTH(@DTE_NAI) > MONTH(@DTE_JOUR) THEN -1
	        WHEN MONTH(@DTE_NAI) < MONTH(@DTE_JOUR) THEN 0
			WHEN DAY(@DTE_NAI) > DAY(@DTE_JOUR) THEN -1
			ELSE 0
	   END 
END
GO

-- La fonction est-elle inlinable
SELECT OBJECT_NAME(object_id) name,
       is_inlineable
FROM sys.sql_modules
WHERE OBJECTPROPERTY(object_id, 'IsScalarFunction') = 1;
GO

--3. Comparaison des performances
---------------------------------------------------------------------
-- calcul de l'ge moyen
-- Avec la fonction non inlineable
DBCC DROPCLEANBUFFERS; ALTER DATABASE SCOPED CONFIGURATION CLEAR PROCEDURE_CACHE;  
GO
SELECT AVG(dbo.fn_get_age(Dt_Naissance)) AS Age FROM Persons  -- 2M de personnes
GO
-- Avec la fonction inlineable
DBCC DROPCLEANBUFFERS; ALTER DATABASE SCOPED CONFIGURATION CLEAR PROCEDURE_CACHE; 
GO
SELECT AVG(dbo.fn_get_age2(Dt_Naissance,CURRENT_TIMESTAMP)) AS Age FROM Persons
--OPTION (USE HINT('DISABLE_TSQL_SCALAR_UDF_INLINING'));
GO
-- Directement dans la requte
DBCC DROPCLEANBUFFERS; ALTER DATABASE SCOPED CONFIGURATION CLEAR PROCEDURE_CACHE;  
GO
SELECT AVG(DATEDIFF(YEAR, Dt_Naissance, CURRENT_TIMESTAMP) +
       CASE WHEN MONTH(Dt_Naissance) > MONTH(CURRENT_TIMESTAMP) THEN -1
	        WHEN MONTH(Dt_Naissance) < MONTH(CURRENT_TIMESTAMP) THEN 0
			WHEN DAY(Dt_Naissance) > DAY(CURRENT_TIMESTAMP) THEN -1
			ELSE 0
	   END) AS Age FROM Persons
GO

----------------------------------
-- Rcursivit : inlinable ?
CREATE OR ALTER FUNCTION dbo.GetNumbersRecurs (@string varchar(350),@pos smallint)
RETURNS varchar(350)
AS
BEGIN
	DECLARE @letter varchar(1), @thenumber varchar(50) = ''
	SET @letter = SUBSTRING(@string,@pos,1)
	IF ISNUMERIC(@letter)=1 and @letter <> '.'
		SET @thenumber = @thenumber+@letter 
    IF @pos < len(@string) AND @pos < 32 -- attention au max recursion
		SET @thenumber = @thenumber + dbo.GetNumbersRecurs (@string,@pos+1) -- rcursif
	RETURN @thenumber
END;
GO
-- La fonction est-elle inlinable
SELECT OBJECT_NAME(object_id) name,definition,is_inlineable FROM sys.sql_modules
GO

-- Voir le plan d'excution
SELECT TOP 30000 Nom, dbo.GetNumbersRecurs(Adresse,1), Adresse FROM Persons
--OPTION (USE HINT('DISABLE_TSQL_SCALAR_UDF_INLINING'));
GO

-- Dsactiver/Activer
--ALTER DATABASE SCOPED CONFIGURATION SET TSQL_SCALAR_UDF_INLINING = OFF
--ALTER DATABASE SCOPED CONFIGURATION SET TSQL_SCALAR_UDF_INLINING = ON


-- Nettoyage
DROP FUNCTION fn_get_age
DROP FUNCTION fn_get_age2
DROP FUNCTION dbo.GetNumbersRecurs
SET STATISTICS TIME OFF