-------------------------------------------------------------
-- Batch mode on row store - SQL Server 2019
-- DATAFLY - 202006
-------------------------------------------------------------
USE AdventureWorksBig
GO
SET STATISTICS TIME ON 
---------------------------------------------------------------------
-- row mode (et donc pas de jointures adaptatives -> uniquement en mode batch, CSI ncessaire !)
---------------------------------------------------------------------
ALTER DATABASE AdventureWorksBig SET COMPATIBILITY_LEVEL = 140
SELECT compatibility_level FROM sys.databases WHERE name = 'AdventureWorksBig'
GO
-- Une requte d'agrgation : 50 M de lignes dans Sales.OrderDetails
DBCC DROPCLEANBUFFERS
ALTER DATABASE SCOPED CONFIGURATION CLEAR PROCEDURE_CACHE;  
SELECT YEAR(SOH.OrderDate) AS [Year],
       PC.Name AS ProductCategory,
       SUM(CAST(SOD.OrderQty AS BIGINT)) AS Quantity
FROM [AdventureWorksBig].[Sales].[SalesOrderDetail] SOD 
    JOIN Sales.SalesOrderHeader SOH 
        ON SOH.SalesOrderID = SOD.SalesOrderID
    JOIN Production.Product P
        ON P.ProductID = SOD.ProductID
    JOIN Production.ProductSubcategory AS PS
        ON PS.ProductSubcategoryID = P.ProductSubcategoryID
    JOIN Production.ProductCategory AS PC
        ON PC.ProductCategoryID = PS.ProductCategoryID
GROUP BY YEAR(SOH.OrderDate),
         PC.Name
ORDER BY YEAR(SOH.OrderDate),
         PC.Name    

-------------------------------------------------------------
-- Avec SQL Server 2019 : batch mode on row store, plus rapide (et jointures adaptatives possibles)
-------------------------------------------------------------
ALTER DATABASE AdventureWorksBig SET COMPATIBILITY_LEVEL = 150
SELECT compatibility_level FROM sys.databases WHERE name = 'AdventureWorksBig'
GO
DBCC DROPCLEANBUFFERS
ALTER DATABASE SCOPED CONFIGURATION CLEAR PROCEDURE_CACHE;  
SELECT YEAR(SOH.OrderDate) AS [Year],
       PC.Name AS ProductCategory,
       SUM(CAST(SOD.OrderQty AS BIGINT)) AS Quantity
FROM [AdventureWorksBig].[Sales].[SalesOrderDetail] SOD 
    JOIN Sales.SalesOrderHeader SOH 
        ON SOH.SalesOrderID = SOD.SalesOrderID
    JOIN Production.Product P
        ON P.ProductID = SOD.ProductID
    JOIN Production.ProductSubcategory AS PS
        ON PS.ProductSubcategoryID = P.ProductSubcategoryID
    JOIN Production.ProductCategory AS PC
        ON PC.ProductCategoryID = PS.ProductCategoryID
GROUP BY YEAR(SOH.OrderDate),
         PC.Name
ORDER BY YEAR(SOH.OrderDate),
         PC.Name    
--OPTION(USE HINT('DISALLOW_BATCH_MODE'));

-- Dsactivation/Activation niveau base
ALTER DATABASE SCOPED CONFIGURATION SET BATCH_MODE_ON_ROWSTORE = OFF;
ALTER DATABASE SCOPED CONFIGURATION SET BATCH_MODE_ON_ROWSTORE = ON;
